package com.arubanetworks.meridiansamples

import android.os.Bundle
import android.util.Log
import com.arubanetworks.meridian.editor.EditorKey
import com.arubanetworks.meridian.maps.MapFragment
import com.arubanetworks.meridian.maps.Transaction
import com.arubanetworks.meridian.requests.MeridianRequest
import com.arubanetworks.meridian.tags.TagBeacon
import com.arubanetworks.meridian.tags.TagMarker
import com.arubanetworks.meridian.tags.TagScan
import com.arubanetworks.meridian.tags.TagStream
import java.util.*


/**
 * A Fragment to demonstrate the use of the TagStream for use in displaying Tags.  Demonstrates how to
 * show all tags for a map or a single tag in a location
 */
class TagsFragment : MapFragment(), TagStream.Listener {

    // A collection to maintain our TagMarkers
    private val tagMarkers = HashMap<TagBeacon, TagMarker>()

    // The TagStream itself
    private var tagStream: TagStream? = null
    private var tagMac : String? = null

    override fun onMapLoadStart() {
        super.onMapLoadStart()
        // Close out any existing tags when loading a new map key
        tagStream?.stopUpdatingTags()

        // Clear any references to old markers so we don't re-add them to the new map on accident
        tagMarkers.clear()

        // Create our TagStream and start it.
        tagStream = tagMac?.let {
            TagStream.Builder()
                    .addTagMac(Application.APP_KEY, it)
                    .setListener(this)
                    .build()
        } ?: run {
            mapView?.mapKey?.let {
                TagStream.Builder()
                        .addMapKey(it)
                        .setListener(this)
                        .build()
            }
        }
    }

    override fun onTagsUpdated(tags: List<TagBeacon>) {

        val transaction = Transaction.Builder().setType(Transaction.Type.UPDATE)

        // Iterate through the new or updated tags and add them to the map.
        for (tag in tags) {
            if (tagMac != null && !tagMac.equals(tag.mac, ignoreCase = true)) continue
            // ignore tags that are not on the current map
            if (mapView != null && mapView.mapKey != null && tag.mapId != mapView.mapKey.id) continue
            var currentMarker: TagMarker? = tagMarkers[tag]

            // check for a new Tag and add
            if (currentMarker == null) {
                if (context == null) {
                    return
                }
                currentMarker = TagMarker(requireContext(), tag)
                tagMarkers[tag] = currentMarker
            } else {
                // Otherwise update our current marker to reflect the new tag state
                val point = tag.location?.point
                point?.let { currentMarker.setPosition(it.x, it.y) }

                // The title or image could also have changed but the TagMarker provided with the
                // SDK doesn't show those fields by default so we can ignore those changes here.
            }

            transaction.addMarker(currentMarker)
        }

        // Commit the transaction
        if (transaction.size > 0)
            mapView.commitTransaction(transaction.build())
    }

    override fun onTagsRemoved(tags: List<TagBeacon>) {
        val transaction = Transaction.Builder().setType(Transaction.Type.REMOVE)

        for (tag in tags) {
            if (tagMac != null && !tagMac.equals(tag.mac, ignoreCase = true)) continue
            // We can ignore removing the tags marker if we never added it.
            val currentMarker = tagMarkers[tag] ?: continue
            transaction.addMarker(currentMarker)
        }

        mapView?.commitTransaction(transaction.build())
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        tagMac = arguments?.getString(TAG_MAC)
    }

    override fun onPause() {
        super.onPause()
        // If we are paused we stop listening to tag updates
        tagStream?.stopUpdatingTags()
    }

    override fun onResume() {
        super.onResume()
        // If we are resuming, start listening for tags again.
        tagStream?.startUpdatingTags()
/*
        // example how to scan for tags that are nearby.
        val ts = TagScan()
        ts.findNearbyTags(
                Application.APP_KEY,
                null, // return all tags
                0, // use default scan length (12s)
                0, // don't filter by RSSI
                MeridianRequest.Listener<List<TagBeacon?>?> { tagBeacons ->
                    if (tagBeacons != null) {
                        Log.d("TagsFragment", "Found " + tagBeacons.size + " nearby tags")
                        for (tag in tagBeacons) {
                            Log.d("TagsFragment", "Found:" + tag.toString())
                        }
                    }
                },
                null
        )*/
    }

    override fun onDestroy() {
        super.onDestroy()
        // Clean up memory.
        tagStream?.dispose()
    }

    companion object {
        private const val TAG_MAC = "meridian.TAG_MAC"
        fun newInstance(mapKey: EditorKey): TagsFragment {
            val f = TagsFragment()
            f.arguments = Builder().setMapKey(mapKey).build().arguments
            return f
        }
        fun newInstance(mapKey: EditorKey, tagMac: String): TagsFragment {
            val f = TagsFragment()
            val args = Builder().setMapKey(mapKey).build().arguments
            if (args != null) {
                args.putString(TAG_MAC, tagMac)
                f.arguments = args
            }
            return f
        }
    }
}
